@@ -511,7 +511,7 @@ errorStateFree(ErrorState * err)
 #if USE_SSL
     delete err->detail;
 #endif
-    cbdataFree(err);
+    delete err;
 }
 
 int
@@ -579,7 +579,7 @@ ErrorState::Dump(MemBuf * mb)
 
     if (ftp.request) {
         str.Printf("FTP Request: %s\r\n", ftp.request);
-        str.Printf("FTP Reply: %s\r\n", ftp.reply);
+        str.Printf("FTP Reply: %s\r\n", (ftp.reply? ftp.reply:"[none]"));
         str.Printf("FTP Msg: ");
         wordlistCat(ftp.server_msg, &str);
         str.Printf("\r\n");
@@ -616,6 +616,12 @@ ErrorState::Convert(char token, bool allowRecursion)
 
         break;
 
+    case 'b':
+        // NP: dynamic location of proxy-port not supported on 3.1
+        // display the generic well-known port instead
+        p = "3128";
+        break;
+
     case 'B':
         p = request ? ftpUrlWith2f(request) : "[no URL]";
 
@@ -670,7 +676,7 @@ ErrorState::Convert(char token, bool allowRecursion)
 
     case 'F':
         /* FTP REPLY LINE */
-        if (ftp.request)
+        if (ftp.reply)
             p = ftp.reply;
         else
             p = "nothing";
@@ -770,7 +776,7 @@ ErrorState::Convert(char token, bool allowRecursion)
                       SQUIDSTRINGPRINT(urlpath_or_slash),
                       request->http_ver.major, request->http_ver.minor);
             packerToMemInit(&pck, &mb);
-            request->header.packInto(&pck);
+            request->header.packInto(&pck, true); //hide authorization data
             packerClean(&pck);
         } else if (request_hdrs) {
             p = request_hdrs;
@@ -836,6 +842,10 @@ ErrorState::Convert(char token, bool allowRecursion)
 
         break;
 
+    case 'x':
+        p = "[Not Available]"; // not supported by 3.1
+        break;
+
     case 'z':
         if (dnsError.size() > 0)
             p = dnsError.termedBuf();
