@@ -100,7 +100,8 @@ extern "C" {
     extern int syslog_enable;	/* 0 */
     extern int theInIcpConnection;	/* -1 */
     extern int theOutIcpConnection;	/* -1 */
-    extern int DnsSocket;		/* -1 */
+    extern int DnsSocketA;		/* -1 */
+    extern int DnsSocketB;		/* -1 */
 #ifdef SQUID_SNMP
 
     extern int theInSnmpConnection;	/* -1 */
@@ -171,10 +172,12 @@ extern "C" {
     extern int ssl_ex_index_server;	/* -1 */
     extern int ssl_ctx_ex_index_dont_verify_domain; /* -1 */
     extern int ssl_ex_index_cert_error_check;	/* -1 */
+    extern int ssl_ex_index_ssl_error_detail;      /* -1 */
 
     extern const char *external_acl_message;      /* NULL */
     extern int opt_send_signal;	/* -1 */
     extern int opt_no_daemon; /* 0 */
+    extern int opt_parse_cfg_only; /* 0 */
 
 
 #ifdef __cplusplus
