@@ -199,7 +199,7 @@ bool IpAddress::IsIPv6() const
 
 bool IpAddress::IsAnyAddr() const
 {
-    return IN6_IS_ADDR_UNSPECIFIED( &m_SocketAddr.sin6_addr ) || IN6_ARE_ADDR_EQUAL( &m_SocketAddr.sin6_addr, &v4_anyaddr);
+    return IN6_IS_ADDR_UNSPECIFIED( &m_SocketAddr.sin6_addr );
 }
 
 /// NOTE: Does NOT clear the Port stored. Ony the Address and Type.
@@ -592,7 +592,12 @@ void IpAddress::GetAddrInfo(struct addrinfo *&dst, int force) const
     memset(dst, 0, sizeof(struct addrinfo));
 
     // set defaults
+    // Mac OS X does not emit a flag indicating the output is numeric (IP address)
+#if _SQUID_APPLE_
+    dst->ai_flags = 0;
+#else
     dst->ai_flags = AI_NUMERICHOST;
+#endif
 
     if (dst->ai_socktype == 0)
         dst->ai_socktype = SOCK_STREAM;
@@ -605,7 +610,7 @@ void IpAddress::GetAddrInfo(struct addrinfo *&dst, int force) const
             && dst->ai_protocol == 0)
         dst->ai_protocol = IPPROTO_UDP;
 
-    if (force == AF_INET6 || (force == AF_UNSPEC && IsIPv6()) ) {
+    if (force == AF_INET6 || (force == AF_UNSPEC && Ip::EnableIpv6 && IsIPv6()) ) {
         dst->ai_addr = (struct sockaddr*)new sockaddr_in6;
 
         memset(dst->ai_addr,0,sizeof(struct sockaddr_in6));
@@ -741,12 +746,14 @@ bool IpAddress::operator <(const IpAddress &rhs) const
     return ( matchIPAddr(rhs) < 0);
 }
 
-u_short IpAddress::GetPort() const
+unsigned short
+IpAddress::GetPort() const
 {
     return ntohs( m_SocketAddr.sin6_port );
 }
 
-u_short IpAddress::SetPort(u_short prt)
+unsigned short
+IpAddress::SetPort(unsigned short prt)
 {
     m_SocketAddr.sin6_port = htons(prt);
 
