@@ -1808,6 +1808,9 @@ SquidShutdown()
     }
 
 #endif
+    // clear StoreController
+    Store::Root(NULL);
+
     fdDumpOpen();
 
     comm_exit();
