@@ -64,7 +64,7 @@ public:
     bool noConnections() const;
     char *url;
     char *host;			/* either request->host or proxy host */
-    u_short port;
+    unsigned short port;
     HttpRequest *request;
     FwdServer *servers;
 
@@ -308,9 +308,14 @@ TunnelStateData::copy (size_t len, comm_err_t errcode, int xerrno, Connection &f
      */
     cbdataInternalLock(this);	/* ??? should be locked by the caller... */
 
-    /* Bump the server connection timeout on any activity */
-    if (!fd_closed(server.fd()))
-        commSetTimeout(server.fd(), Config.Timeout.read, tunnelTimeout, this);
+    /* Bump the source connection timeout on any activity */
+    if (!fd_closed(from.fd()))
+        commSetTimeout(from.fd(), Config.Timeout.read, tunnelTimeout, this);
+
+    /* Bump the dest connection read timeout on any activity */
+    /* see Bug 3659: tunnels can be weird, with very long one-way transfers */
+    if (!fd_closed(to.fd()))
+        commSetTimeout(to.fd(), Config.Timeout.read, tunnelTimeout, this);
 
     if (errcode)
         from.error (xerrno);
@@ -470,7 +475,7 @@ tunnelConnectTimeout(int fd, void *data)
     if (tunnelState->servers) {
         if (tunnelState->servers->_peer)
             hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
-                          tunnelState->servers->_peer->host);
+                          tunnelState->servers->_peer->name);
         else if (Config.onoff.log_ip_on_direct)
             hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
                           fd_table[tunnelState->server.fd()].ipaddr);
@@ -542,7 +547,7 @@ tunnelErrorComplete(int fdnotused, void *data, size_t sizenotused)
     if (!fd_closed(tunnelState->client.fd()))
         comm_close(tunnelState->client.fd());
 
-    if (fd_closed(tunnelState->server.fd()))
+    if (!fd_closed(tunnelState->server.fd()))
         comm_close(tunnelState->server.fd());
 
     cbdataInternalUnlock(tunnelState);
@@ -560,7 +565,7 @@ tunnelConnectDone(int fdnotused, const DnsLookupDetails &dns, comm_err_t status,
 
     if (tunnelState->servers->_peer)
         hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
-                      tunnelState->servers->_peer->host);
+                      tunnelState->servers->_peer->name);
     else if (Config.onoff.log_ip_on_direct)
         hierarchyNote(&tunnelState->request->hier, tunnelState->servers->code,
                       fd_table[tunnelState->server.fd()].ipaddr);
