@@ -1,6 +1,6 @@
 
 /*
- * $Id: mem_hdr_test.cc,v 1.6 2007/08/15 04:08:30 amosjeffries Exp $
+ * $Id$
  *
  * DEBUG: section 19    Store Memory Primitives
  * AUTHOR: Robert Collins
@@ -37,9 +37,12 @@
 #include "squid.h"
 #include "stmem.h"
 #include "mem_node.h"
-#include <iostream>
 #include "Generic.h"
 
+#if HAVE_IOSTREAM
+#include <iostream>
+#endif
+
 void
 testLowAndHigh()
 {
@@ -47,7 +50,7 @@ testLowAndHigh()
     assert (aHeader.lowestOffset() == 0);
     assert (aHeader.write (StoreIOBuffer()));
     assert (aHeader.lowestOffset() == 0);
-    assert (aHeader.write (StoreIOBuffer(0, 1, NULL)));
+    assert (aHeader.write (StoreIOBuffer(0, 1, (char *)NULL)));
     assert (aHeader.lowestOffset() == 0);
     char * sampleData = xstrdup ("A");
     assert (aHeader.write (StoreIOBuffer(1, 100, sampleData)));
